@@ -322,7 +322,7 @@ public:
 #endif
 
   /* See RANGE_OPT_PARAM::alloced_sel_args */
-  enum { MAX_SEL_ARGS = 16000 };
+  enum { DEFAULT_MAX_SEL_ARGS = 16000 };
 
   SEL_ARG() = default;
   SEL_ARG(SEL_ARG &);
@@ -713,6 +713,8 @@ public:
   */
   bool remove_false_where_parts;
 
+  bool note_unusable_keys;        // Give SQL notes for unusable keys
+
   /*
     used_key_no -> table_key_no translation table. Only makes sense if
     using_real_indexes==TRUE
@@ -736,8 +738,9 @@ public:
   {
     return
       thd->killed ||
+      thd->is_fatal_error ||
       thd->is_error() ||
-      alloced_sel_args > SEL_ARG::MAX_SEL_ARGS;
+      alloced_sel_args > thd->variables.optimizer_max_sel_args;
   }
 };
 
@@ -1161,28 +1164,28 @@ public:
     { return new QUICK_RANGE_SELECT(thd, head, index, no_alloc, parent_alloc,
                                     create_error); }
   
-  void need_sorted_output() override;
-  int init() override;
-  int reset(void) override;
-  int get_next() override;
-  void range_end() override;
+  void need_sorted_output();
+  int init();
+  int reset(void);
+  int get_next();
+  void range_end();
   int get_next_prefix(uint prefix_length, uint group_key_parts, 
                       uchar *cur_prefix);
-  bool reverse_sorted() override { return 0; }
-  bool unique_key_range() override;
-  int init_ror_merged_scan(bool reuse_handler, MEM_ROOT *alloc) override;
-  void save_last_pos() override
+  bool reverse_sorted() { return 0; }
+  bool unique_key_range();
+  int init_ror_merged_scan(bool reuse_handler, MEM_ROOT *alloc);
+  void save_last_pos()
   { file->position(record); }
-  int get_type() override { return QS_TYPE_RANGE; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
+  int get_type() { return QS_TYPE_RANGE; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
 #ifndef DBUG_OFF
-  void dbug_dump(int indent, bool verbose) override;
+  void dbug_dump(int indent, bool verbose);
 #endif
-  void replace_handler(handler *new_file) override { file= new_file; }
-  QUICK_SELECT_I *make_reverse(uint used_key_parts_arg) override;
+  virtual void replace_handler(handler *new_file) { file= new_file; }
+  QUICK_SELECT_I *make_reverse(uint used_key_parts_arg);
 
-  void add_used_key_part_to_set() override;
+  virtual void add_used_key_part_to_set();
 
 private:
   /* Default copy ctor used by QUICK_SELECT_DESC */
@@ -1230,13 +1233,13 @@ public:
     :QUICK_RANGE_SELECT(thd, table, index_arg, no_alloc, parent_alloc,
     create_err)
     {};
-  QUICK_RANGE_SELECT *clone(bool *create_error) override
+  virtual QUICK_RANGE_SELECT *clone(bool *create_error)
     {
       DBUG_ASSERT(0);
       return new QUICK_RANGE_SELECT_GEOM(thd, head, index, no_alloc,
                                          parent_alloc, create_error);
     }
-  int get_next() override;
+  virtual int get_next();
 };
 
 
@@ -1312,16 +1315,16 @@ public:
   QUICK_INDEX_SORT_SELECT(THD *thd, TABLE *table);
   ~QUICK_INDEX_SORT_SELECT();
 
-  int  init() override;
-  void need_sorted_output() override { DBUG_ASSERT(0); /* Can't do it */ }
-  int  reset(void) override;
-  bool reverse_sorted() override { return false; }
-  bool unique_key_range() override { return false; }
-  bool is_keys_used(const MY_BITMAP *fields) override;
+  int  init();
+  void need_sorted_output() { DBUG_ASSERT(0); /* Can't do it */ }
+  int  reset(void);
+  bool reverse_sorted() { return false; }
+  bool unique_key_range() { return false; }
+  bool is_keys_used(const MY_BITMAP *fields);
 #ifndef DBUG_OFF
-  void dbug_dump(int indent, bool verbose) override;
+  void dbug_dump(int indent, bool verbose);
 #endif
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
 
   bool push_quick_back(QUICK_RANGE_SELECT *quick_sel_range);
 
@@ -1333,7 +1336,7 @@ public:
 
   MEM_ROOT alloc;
   THD *thd;
-  bool is_valid() override
+  virtual bool is_valid()
   {
     List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
     QUICK_RANGE_SELECT *quick;
@@ -1352,7 +1355,7 @@ public:
   /* used to get rows collected in Unique */
   READ_RECORD read_record;
 
-  void add_used_key_part_to_set() override;
+  virtual void add_used_key_part_to_set();
 };
 
 
@@ -1363,31 +1366,31 @@ private:
   /* true if this select is currently doing a clustered PK scan */
   bool  doing_pk_scan;
 protected:
-  int read_keys_and_merge() override;
+  int read_keys_and_merge();
 
 public:
   QUICK_INDEX_MERGE_SELECT(THD *thd_arg, TABLE *table)
     :QUICK_INDEX_SORT_SELECT(thd_arg, table) {}
 
-  int get_next() override;
-  int get_type() override { return QS_TYPE_INDEX_MERGE; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
+  int get_next();
+  int get_type() { return QS_TYPE_INDEX_MERGE; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
 };
 
 class QUICK_INDEX_INTERSECT_SELECT : public QUICK_INDEX_SORT_SELECT
 {
 protected:
-  int read_keys_and_merge() override;
+  int read_keys_and_merge();
 
 public:
   QUICK_INDEX_INTERSECT_SELECT(THD *thd_arg, TABLE *table)
     :QUICK_INDEX_SORT_SELECT(thd_arg, table) {}
 
   key_map filtered_scans;
-  int get_next() override;
-  int get_type() override { return QS_TYPE_INDEX_INTERSECT; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
+  int get_next();
+  int get_type() { return QS_TYPE_INDEX_INTERSECT; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
 };
 
 
@@ -1417,21 +1420,21 @@ public:
                              MEM_ROOT *parent_alloc);
   ~QUICK_ROR_INTERSECT_SELECT();
 
-  int  init() override;
-  void need_sorted_output() override { DBUG_ASSERT(0); /* Can't do it */ }
-  int  reset(void) override;
-  int  get_next() override;
-  bool reverse_sorted() override { return false; }
-  bool unique_key_range() override { return false; }
-  int get_type() override { return QS_TYPE_ROR_INTERSECT; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
-  bool is_keys_used(const MY_BITMAP *fields) override;
-  void add_used_key_part_to_set() override;
+  int  init();
+  void need_sorted_output() { DBUG_ASSERT(0); /* Can't do it */ }
+  int  reset(void);
+  int  get_next();
+  bool reverse_sorted() { return false; }
+  bool unique_key_range() { return false; }
+  int get_type() { return QS_TYPE_ROR_INTERSECT; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
+  bool is_keys_used(const MY_BITMAP *fields);
+  void add_used_key_part_to_set();
 #ifndef DBUG_OFF
-  void dbug_dump(int indent, bool verbose) override;
+  void dbug_dump(int indent, bool verbose);
 #endif
-  int init_ror_merged_scan(bool reuse_handler, MEM_ROOT *alloc) override;
+  int init_ror_merged_scan(bool reuse_handler, MEM_ROOT *alloc);
   bool push_quick_back(MEM_ROOT *alloc, QUICK_RANGE_SELECT *quick_sel_range);
 
   class QUICK_SELECT_WITH_RECORD : public Sql_alloc
@@ -1448,7 +1451,7 @@ public:
   */
   List<QUICK_SELECT_WITH_RECORD> quick_selects;
 
-  bool is_valid() override
+  virtual bool is_valid()
   {
     List_iterator_fast<QUICK_SELECT_WITH_RECORD> it(quick_selects);
     QUICK_SELECT_WITH_RECORD *quick;
@@ -1497,26 +1500,26 @@ public:
   QUICK_ROR_UNION_SELECT(THD *thd, TABLE *table);
   ~QUICK_ROR_UNION_SELECT();
 
-  int  init() override;
-  void need_sorted_output() override { DBUG_ASSERT(0); /* Can't do it */ }
-  int  reset(void) override;
-  int  get_next() override;
-  bool reverse_sorted() override { return false; }
-  bool unique_key_range() override { return false; }
-  int get_type() override { return QS_TYPE_ROR_UNION; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
-  bool is_keys_used(const MY_BITMAP *fields) override;
-  void add_used_key_part_to_set() override;
+  int  init();
+  void need_sorted_output() { DBUG_ASSERT(0); /* Can't do it */ }
+  int  reset(void);
+  int  get_next();
+  bool reverse_sorted() { return false; }
+  bool unique_key_range() { return false; }
+  int get_type() { return QS_TYPE_ROR_UNION; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
+  bool is_keys_used(const MY_BITMAP *fields);
+  void add_used_key_part_to_set();
 #ifndef DBUG_OFF
-  void dbug_dump(int indent, bool verbose) override;
+  void dbug_dump(int indent, bool verbose);
 #endif
 
   bool push_quick_back(QUICK_SELECT_I *quick_sel_range);
 
   List<QUICK_SELECT_I> quick_selects; /* Merged quick selects */
 
-  bool is_valid() override
+  virtual bool is_valid()
   {
     List_iterator_fast<QUICK_SELECT_I> it(quick_selects);
     QUICK_SELECT_I *quick;
@@ -1589,6 +1592,7 @@ private:
   uchar *group_prefix;    /* Key prefix consisting of the GROUP fields. */
   const uint group_prefix_len; /* Length of the group prefix. */
   uint group_key_parts;  /* A number of keyparts in the group prefix */
+  uchar *last_prefix;     /* Prefix of the last group for detecting EOF. */
   bool have_min;         /* Specify whether we are computing */
   bool have_max;         /*   a MIN, a MAX, or both.         */
   bool have_agg_distinct;/*   aggregate_function(DISTINCT ...).  */
@@ -1642,21 +1646,21 @@ public:
   void update_key_stat();
   void adjust_prefix_ranges();
   bool alloc_buffers();
-  int init() override;
-  void need_sorted_output() override { /* always do it */ }
-  int reset() override;
-  int get_next() override;
-  bool reverse_sorted() override { return false; }
-  bool unique_key_range() override { return false; }
-  int get_type() override { return QS_TYPE_GROUP_MIN_MAX; }
-  void add_keys_and_lengths(String *key_names, String *used_lengths) override;
-  void add_used_key_part_to_set() override;
+  int init();
+  void need_sorted_output() { /* always do it */ }
+  int reset();
+  int get_next();
+  bool reverse_sorted() { return false; }
+  bool unique_key_range() { return false; }
+  int get_type() { return QS_TYPE_GROUP_MIN_MAX; }
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  void add_used_key_part_to_set();
 #ifndef DBUG_OFF
-  void dbug_dump(int indent, bool verbose) override;
+  void dbug_dump(int indent, bool verbose);
 #endif
   bool is_agg_distinct() { return have_agg_distinct; }
   bool loose_scan_is_scanning() { return is_index_scan; }
-  Explain_quick_select *get_explain(MEM_ROOT *alloc) override;
+  Explain_quick_select *get_explain(MEM_ROOT *alloc);
 };
 
 
@@ -1664,18 +1668,18 @@ class QUICK_SELECT_DESC: public QUICK_RANGE_SELECT
 {
 public:
   QUICK_SELECT_DESC(QUICK_RANGE_SELECT *q, uint used_key_parts);
-  QUICK_RANGE_SELECT *clone(bool *create_error) override
+  virtual QUICK_RANGE_SELECT *clone(bool *create_error)
     { DBUG_ASSERT(0); return new QUICK_SELECT_DESC(this, used_key_parts); }
-  int get_next() override;
-  bool reverse_sorted() override { return 1; }
-  int get_type() override { return QS_TYPE_RANGE_DESC; }
-  QUICK_SELECT_I *make_reverse(uint used_key_parts_arg) override
+  int get_next();
+  bool reverse_sorted() { return 1; }
+  int get_type() { return QS_TYPE_RANGE_DESC; }
+  QUICK_SELECT_I *make_reverse(uint used_key_parts_arg)
   {
     return this; // is already reverse sorted
   }
 private:
   bool range_reads_after_key(QUICK_RANGE *range);
-  int reset(void) override { rev_it.rewind(); return QUICK_RANGE_SELECT::reset(); }
+  int reset(void) { rev_it.rewind(); return QUICK_RANGE_SELECT::reset(); }
   List<QUICK_RANGE> rev_ranges;
   List_iterator<QUICK_RANGE> rev_it;
   uint used_key_parts;
@@ -1708,13 +1712,20 @@ class SQL_SELECT :public Sql_alloc {
   ~SQL_SELECT();
   void cleanup();
   void set_quick(QUICK_SELECT_I *new_quick) { delete quick; quick= new_quick; }
+
+  /*
+    @return
+      true  - for ERROR and IMPOSSIBLE_RANGE
+      false   - Ok
+  */
   bool check_quick(THD *thd, bool force_quick_range, ha_rows limit)
   {
     key_map tmp;
     tmp.set_all();
     return test_quick_select(thd, tmp, 0, limit, force_quick_range,
-                             FALSE, FALSE, FALSE) < 0;
+                             FALSE, FALSE, FALSE) != OK;
   }
+
   /* 
     RETURN
       0   if record must be skipped <-> (cond && cond->val_int() == 0)
@@ -1728,12 +1739,25 @@ class SQL_SELECT :public Sql_alloc {
       rc= -1;
     return rc;
   }
-  int test_quick_select(THD *thd, key_map keys, table_map prev_tables,
-			ha_rows limit, bool force_quick_range, 
-                        bool ordered_output, bool remove_false_parts_of_where,
-                        bool only_single_index_range_scan);
+
+  enum quick_select_return_type {
+    IMPOSSIBLE_RANGE = -1,
+    ERROR,
+    OK
+  };
+
+  enum quick_select_return_type
+  test_quick_select(THD *thd, key_map keys, table_map prev_tables,
+                    ha_rows limit,
+                    bool force_quick_range,
+                    bool ordered_output,
+                    bool remove_false_parts_of_where,
+                    bool only_single_index_range_scan,
+                    bool suppress_unusable_key_notes = 0);
 };
 
+typedef enum SQL_SELECT::quick_select_return_type quick_select_return;
+
 
 class SQL_SELECT_auto
 {
@@ -1774,12 +1798,12 @@ public:
       QUICK_RANGE_SELECT (thd, table, key, 1, NULL, create_err) 
   { (void) init(); }
   ~FT_SELECT() { file->ft_end(); }
-  QUICK_RANGE_SELECT *clone(bool *create_error) override
+  virtual QUICK_RANGE_SELECT *clone(bool *create_error)
     { DBUG_ASSERT(0); return new FT_SELECT(thd, head, index, create_error); }
-  int init() override { return file->ft_init(); }
-  int reset() override { return 0; }
-  int get_next() override { return file->ha_ft_read(record); }
-  int get_type() override { return QS_TYPE_FULLTEXT; }
+  int init() { return file->ft_init(); }
+  int reset() { return 0; }
+  int get_next() { return file->ha_ft_read(record); }
+  int get_type() { return QS_TYPE_FULLTEXT; }
 };
 
 FT_SELECT *get_ft_select(THD *thd, TABLE *table, uint key);
